
#ifndef _burn_H_
#define _burn_H_

#include "config.h"

#include <kio/slavebase.h>
#include <kio/jobclasses.h>
#include <kprogress.h>
#include <qvaluelist.h>
#include <qfile.h>
#include <kfileitem.h>

#include <akode/audioframe.h>
#include <akode/framedecoder.h>

#include <audiofile.h>

class KProcess;
class kio_burnProtocol;
class KConfig;
class KURL;
class ProgressDlg;
class BurnDiscWizard;

typedef struct {
  
  bool blankCDRW;
  bool blankOnly;
  bool advSettings;
  bool doNotFixate;
  QString burnSpeed;
  enum burnMode{ TAO, SAO } mode;
  enum discType{ DataCD, AudioCD } type;
} BurnOptions;

typedef enum { mkisofs, cdrecord, decodeaudio } Stage;

/**IOSlave implementation for accessing burn:/

This implements the slave base class from the KIO library, which allows it to
be loaded dynamically by kde programs.*/ 
class kio_burnProtocol : public QObject, public KIO::SlaveBase
{
  Q_OBJECT
  
  public:
    /**Pretty standard reimplementation from SlaveBase*/
    kio_burnProtocol(const QCString &pool_socket, const QCString &app_socket);
    virtual ~kio_burnProtocol();
    
    /*These functions reimplement standard services from slavebase*/
    virtual void mimetype(const KURL& url);
    virtual void get(const KURL& url);
    virtual void listDir(const KURL& url);
    virtual void stat(const KURL& url);
    virtual void put( const KURL& url, int permissions, bool overwrite, bool resume );
    virtual void del( const KURL & url, bool isFile );
    virtual void rename( const KURL& src, const KURL& dest, bool overwrite);
    virtual void symlink( const QString& target, const KURL& dest, bool overwrite);
    virtual void chmod( const KURL &url, int permissions );
    virtual void mkdir( const KURL &url, int permissions );
    virtual void rmdir( const KURL &url );
 
    /**Returns an error message to the job saying that the user cancelled the operation*/
    void error_user_cancel();
    
    /**Activated when user clicks on "burn disc", calls up disc burning UI and 
      executes cdrecord and friends*/
    void burnDisc();
    
  private slots:
  
    /*These functions are used to pass info acquired from the file: slave back the the job
    that is controlling this slave. (For accessing stuff in the Data/ directory.)*/
    /**Called whenever a job we are using returns a totalSize to us (e.g. in listDir or get)*/
    void slotTotalSize(  KIO::Job *, KIO::filesize_t size);
    /**Called whenever a mime type job ends*/
    void mimeTypeJobFinished( KIO::Job * job );
    /**When data is recieved from the file IO slave for a get job in the Data directory, this
    slot passes it on with data()*/
    void getJobData( KIO::Job * job, const QByteArray &data  );
    /**A slot to be called when a job that requires no special action finishes. All this does
    is check @p job for errors and terminates the Qt event loop that we had to start to
    recieve signals.*/
    void simpleFinish( KIO::Job * job );
    void statJobFinished( KIO::Job * job );
    void listJobFinished( KIO::Job * job );
    void listJobEntries(KIO::Job *job, const KIO::UDSEntryList &list); 
    void putDataReq( KIO::Job *, QByteArray &data );

    
    /**Cleans up and checks for errors when one of the burning processes finished*/
    void slotProcessExited( KProcess *process );   
    /**Reads the output of mkisofs and uses it to control the progress bar in the progress dialog*/
    void mkisofsStdout( KProcess *process, char * buffer, int buflen );
    /**Like mkisofsStdout but for cdrecord*/
    void cdrecordStdout( KProcess *process, char * buffer, int buflen );
    /**Receives output from growisofs and uses it to control UI, see mkisofStdout() and cdrecordStdout()*/
    void growisofsStdout( KProcess*, char*, int );  
    
    /**Simply calls qApp->exit_loop(), used to allow signals to cause a halt in the
      event loop*/
    void exitQAppLoop();

    /**Utility function for burndisc - decodes one frame of audio data (from m_decoder) to
    wav file (in m_afhandle). If m_decoder is at eof, will emit decoderFinished signal.*/
    void decodeToWav();
        
  signals:
    void decoderFinished();
    
  private:
    KIO::StatJob *m_statJob;
    KIO::ListJob *m_listJob;
    KIO::TransferJob *m_transferJob;
    KIO::MimetypeJob *m_mimeTypeJob;
    KIO::DeleteJob *m_deleteJob;
    KIO::SimpleJob *m_simpleJob;
    KProcess *m_process;
    
    KConfig *m_config;
    
    BurnDiscWizard *m_burnDiscWizard;
    ProgressDlg *m_progressDialog;  
    
    QString *m_data_dir;
    QString *m_audio_dir;
    QRegExp *m_data_rx;
    QRegExp *m_data_rxx;
    
    int m_scsibus, m_target, m_lun, m_isAtapi;
    
    bool m_cdrecordFailed, m_notDcop;
    
    /**Stores the dcop appid of the konqueror plugin that might be calling us*/
    QString m_dcopId;
    
    BurnOptions m_options;
    
    QFile *m_cdrecordLog;
    
    aKode::FrameDecoder *m_decoder;
    AFfilehandle m_afhandle;

    Stage m_burnStage;
    
    enum ResultTemplate { Success , Failure, SettingsChanged, SettingsNotChanged } ;
    
    /**Turns HTML file template into QByteArray*/
    void getHTMLByteArray( ResultTemplate t, QByteArray &dest ) const;
    /**Returns the current working directory as a char * - mainly a utility
    function used by the burnDisc function*/
    static char * cwd();
    /**Set the text for the progress dialog, this will either be m_progressDialog or
    a message sent to the instance of konqueror identified by m_dcopId*/
    void setProgressBarText( QString text );
    /**Set the total number of steps for the progress dialog, this will either be m_progressDialog or
    a message sent to the instance of konqueror identified by m_dcopId*/
    void setProgressBarTotal( int total );
    /**Set the progress for the progress dialog, this will either be m_progressDialog or
    a message sent to the instance of konqueror identified by m_dcopId*/
    void setProgressBarProgress( int progress );
    void setProgressBarShown( bool show );
};


#endif
