/***************************************************************************
 *   Copyright (C) 2005 by James Thorniley                                 *
 *   james@uncommonlygood.co.uk                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BURNDISCWIZARD_H
#define BURNDISCWIZARD_H

#include <kwizard.h>
#include <kfileitem.h>

class BurnOptionsPage;
class AdvancedSettingsDialog;
class PlaylistPage;
class KConfig;
class QStringList;
class QString;
class QLabel;
class KProcess;
class DriveSelectPage;
class ScanMediaPage;
class QTimer;
class ScanBusThread;
class KDirLister;
class QCloseEvent;

/**
\brief Wizard for burning discs.
@par
Consists of an options page, followed (if we are creating an Audio CD) by a playlist editing page,
followed by a final page where the current media in the selected drive will be scanned. If the media is
invalid the user will be invited to change it to start the burning process.
@par
If no burning device has been selected before instatiating the BurnDiscWizard (that is the "file" entry in the
KConfig object given to the constructor is null), a drive selection page will be presented to the user first.

@author James Thorniley
*/
class BurnDiscWizard : public KWizard
{
  Q_OBJECT
  public:
    /**Create a new BurnDiscWizard.
    
    @param config Will be used to load and store settings such as which device should be used,
    the order of an audio CD playlist, and the "advanced" settings like burning speed.*/
    BurnDiscWizard( KConfig *config, QWidget *parent = 0, const char *name = 0 );

    ~BurnDiscWizard();
    
    BurnOptionsPage * optionsPage() const { return m_burnOptionsPage; }
    AdvancedSettingsDialog * advancedSettingsDialog() const { return m_advancedSettingsDialog; }
    PlaylistPage *playlistPage() const { return m_playlistPage; }
    
    /**returns whether we think the current media is a DVD format. Be careful not to use this before
    the media has been scanned (i.e. checkScanFinished() has been called at least once with 
    m_scanBusThread->finished() == true)*/
    bool mediaIsDVD() const { return m_mediaIsDVD; }
    /**Returns whether the inserted media is rewritable (DVD+-RW or CDRW). Like mediaIsDVD(), should only
    be used after scan is finished.*/
    bool mediaIsRW() const { return m_mediaIsReWritable; }      
    
  private slots:
    /**Sets the playlist editing page as appropriate if the user selects audio CD
    @param type The type of disc.*/
    void discTypeChanged( int type );
    
    void moveTrackUp();
    void moveTrackDown();  
    
    void addItemsToTrackList( const QStringList &items );
    
    void renumberTracks();
    
    //if title == i18n( "Burn Options" ) scan the current drive for available burning speeds
    void pageChanged( const QString &title ); //if title == i18n("Playlist") scan audio files etc
    
    void exitQAppLoop() const;
    
    void createTrackListing();
    void addItemsToScannedList( const KFileItemList &items );
    
    void speedProcessExited( KProcess * );
    void speedProcessStdout( KProcess */*proc*/, char *buffer, int buflen );
    
    void scanMedia();
    void checkScanFinished();
    
  protected:
    virtual void closeEvent(QCloseEvent *e);
  private:
    KProcess *m_speedProc;
    KConfig *m_config;
    BurnOptionsPage *m_burnOptionsPage;
    AdvancedSettingsDialog *m_advancedSettingsDialog;
    PlaylistPage *m_playlistPage;
    DriveSelectPage *m_settingsPage;
    ScanMediaPage *m_scanPage;
    ScanBusThread *m_scanBusThread;
    
    KDirLister *m_dirLister;
    
    QStringList *m_scannedList, *m_outputList, *m_savedList;
    
    QTimer *m_timer;
    
    bool m_mediaIsReWritable;
    bool m_mediaIsDVD;
};

#endif
